"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DIProvider = void 0;
const jsx_runtime_1 = require("react/jsx-runtime");
const cheap_di_1 = require("cheap-di");
const react_1 = require("react");
const DiContext_js_1 = require("../DiContext.js");
const index_js_1 = require("../hooks/index.js");
const InternalLogger_js_1 = require("../InternalLogger.js");
const DIProvider = (props) => {
    const { dependencies, self, selfSingletons, parentContainer, debugName, children } = props;
    const [logger] = (0, react_1.useState)(() => new InternalLogger_js_1.InternalLogger(debugName));
    const [initialized, setInitialized] = (0, react_1.useState)(false);
    const timerRef = (0, react_1.useRef)(null);
    const diContext = (0, index_js_1.useDiContext)({ logger, parentContainer });
    const { container: reactContainer } = diContext;
    (0, react_1.useEffect)(() => {
        const isAnyConfigurationPassed = dependencies || self || selfSingletons || parentContainer;
        if (!reactContainer || !isAnyConfigurationPassed) {
            return;
        }
        reactContainer.clear();
        logger.log('dependency registrations');
        const singletonsSizeBeforeDependenciesUpdate = reactContainer?.getSingletons().size ?? 0;
        dependencies?.forEach((dependency) => dependency(reactContainer));
        self?.forEach((selfDependency) => reactContainer.registerImplementation(selfDependency));
        selfSingletons?.forEach((selfDependency) => reactContainer.registerImplementation(selfDependency).asSingleton());
        logger.log('singleton and stateful configurations');
        for (const [type] of reactContainer.getDependencies()) {
            const implementation = reactContainer.localScope((container) => container.getImplementation(type));
            if (!implementation) {
                continue;
            }
            if ((0, cheap_di_1.isSingleton)(implementation)) {
                logger.log('singleton', implementation, 'founded');
                // instantiate singleton in local scope (it may use local scope dependencies)
                reactContainer.resolve(type);
            }
        }
        if (reactContainer.parentContainer &&
            singletonsSizeBeforeDependenciesUpdate !== reactContainer.getSingletons().size) {
            logger.log('singletons size changed, trigger root rerender');
            timerRef.current = setTimeout(() => {
                reactContainer.rootReactContainer.rerender();
            });
        }
        setInitialized(true);
    }, [reactContainer, dependencies, self, selfSingletons]);
    (0, react_1.useEffect)(() => {
        return () => {
            if (timerRef.current) {
                clearTimeout(timerRef.current);
            }
            reactContainer?.clear();
        };
    }, []);
    if (!initialized) {
        return null;
    }
    return (0, jsx_runtime_1.jsx)(DiContext_js_1.DiContext.Provider, { value: diContext, children: children });
};
exports.DIProvider = DIProvider;
//# sourceMappingURL=DIProvider.js.map