"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ReactContainer = void 0;
const cheap_di_1 = require("cheap-di");
class ReactContainer extends cheap_di_1.ContainerImpl {
    parentContainer;
    rerender;
    scope;
    skipParentInstanceOnce;
    constructor(parentContainer) {
        super();
        this.parentContainer = parentContainer;
        this.rerender = () => undefined;
        this.scope = 'global';
        this.skipParentInstanceOnce = false;
    }
    get rootContainer() {
        return this.findRootContainer();
    }
    get rootReactContainer() {
        return this.findReactContainerContainer();
    }
    sameParent(parentContainer) {
        return this.parentContainer === parentContainer;
    }
    getDependencies() {
        return this.dependencies;
    }
    localScope(callback) {
        this.scope = 'local';
        const result = callback(this);
        this.scope = 'global';
        return result;
    }
    skipParentInstanceResolvingOnce() {
        this.skipParentInstanceOnce = true;
    }
    getInstance(type) {
        if (this.instances.has(type)) {
            return this.instances.get(type);
        }
        if (this.parentContainer && this.scope === 'global') {
            if (this.skipParentInstanceOnce) {
                this.skipParentInstanceOnce = false;
                return undefined;
            }
            return this.parentContainer.getInstance(type);
        }
        return undefined;
    }
    getImplementation(type) {
        if (this.dependencies.has(type)) {
            return this.dependencies.get(type);
        }
        if (this.parentContainer && this.scope === 'global') {
            return this.parentContainer.getImplementation(type);
        }
        return undefined;
    }
    getSingletons() {
        const rootContainer = this.findRootContainer();
        return rootContainer.singletons;
    }
    findRootContainer() {
        if (this.parentContainer) {
            if (this.parentContainer instanceof ReactContainer) {
                return this.parentContainer.findRootContainer();
            }
            return this.parentContainer;
        }
        return this;
    }
    findReactContainerContainer() {
        if (this.parentContainer instanceof ReactContainer) {
            return this.parentContainer.findReactContainerContainer();
        }
        return this;
    }
}
exports.ReactContainer = ReactContainer;
//# sourceMappingURL=ReactContainer.js.map