"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useDiContext = void 0;
const react_1 = require("react");
const DiContext_js_1 = require("../DiContext.js");
const ReactContainer_js_1 = require("../ReactContainer.js");
function useDiContext(params = {}) {
    const { logger = console } = params;
    const parentContainer = (0, react_1.useContext)(DiContext_js_1.DiContext)?.container;
    const [contextValue, setContextValue] = (0, react_1.useState)({ container: undefined });
    (0, react_1.useEffect)(() => {
        const parent = parentContainer ?? params.parentContainer;
        if (!contextValue.container) {
            if (parent) {
                logger.log('create container');
                contextValue.container = new ReactContainer_js_1.ReactContainer(parent);
            }
            else {
                logger.log('create root container');
                contextValue.container = new ReactContainer_js_1.ReactContainer();
                contextValue.container.rerender = () => setContextValue({ ...contextValue });
            }
            setContextValue({ container: contextValue.container });
        }
        if (!parent || contextValue.container.sameParent(parent)) {
            return;
        }
        // exceptional case: if by some reasons parent container will be changed
        // possible, unreachable case
        logger.log('RECREATE container');
        setContextValue({ container: new ReactContainer_js_1.ReactContainer(parent) });
    }, [parentContainer, params.parentContainer]);
    return contextValue;
}
exports.useDiContext = useDiContext;
//# sourceMappingURL=useDiContext.js.map