import { jsx as _jsx } from "react/jsx-runtime";
import { isSingleton } from 'cheap-di';
import { useEffect, useRef, useState } from 'react';
import { DiContext } from '../DiContext.js';
import { useDiContext } from '../hooks/index.js';
import { InternalLogger } from '../InternalLogger.js';
const DIProvider = (props) => {
    const { dependencies, self, selfSingletons, parentContainer, debugName, children } = props;
    const [logger] = useState(() => new InternalLogger(debugName));
    const [initialized, setInitialized] = useState(false);
    const timerRef = useRef(null);
    const diContext = useDiContext({ logger, parentContainer });
    const { container: reactContainer } = diContext;
    useEffect(() => {
        const isAnyConfigurationPassed = dependencies || self || selfSingletons || parentContainer;
        if (!reactContainer || !isAnyConfigurationPassed) {
            return;
        }
        reactContainer.clear();
        logger.log('dependency registrations');
        const singletonsSizeBeforeDependenciesUpdate = reactContainer?.getSingletons().size ?? 0;
        dependencies?.forEach((dependency) => dependency(reactContainer));
        self?.forEach((selfDependency) => reactContainer.registerImplementation(selfDependency));
        selfSingletons?.forEach((selfDependency) => reactContainer.registerImplementation(selfDependency).asSingleton());
        logger.log('singleton and stateful configurations');
        for (const [type] of reactContainer.getDependencies()) {
            const implementation = reactContainer.localScope((container) => container.getImplementation(type));
            if (!implementation) {
                continue;
            }
            if (isSingleton(implementation)) {
                logger.log('singleton', implementation, 'founded');
                // instantiate singleton in local scope (it may use local scope dependencies)
                reactContainer.resolve(type);
            }
        }
        if (reactContainer.parentContainer &&
            singletonsSizeBeforeDependenciesUpdate !== reactContainer.getSingletons().size) {
            logger.log('singletons size changed, trigger root rerender');
            timerRef.current = setTimeout(() => {
                reactContainer.rootReactContainer.rerender();
            });
        }
        setInitialized(true);
    }, [reactContainer, dependencies, self, selfSingletons]);
    useEffect(() => {
        return () => {
            if (timerRef.current) {
                clearTimeout(timerRef.current);
            }
            reactContainer?.clear();
        };
    }, []);
    if (!initialized) {
        return null;
    }
    return _jsx(DiContext.Provider, { value: diContext, children: children });
};
export { DIProvider };
//# sourceMappingURL=DIProvider.js.map