import { useContext, useEffect, useState } from 'react';
import { DiContext } from '../DiContext.js';
import { ReactContainer } from '../ReactContainer.js';
export function useDiContext(params = {}) {
    const { logger = console } = params;
    const parentContainer = useContext(DiContext)?.container;
    const [contextValue, setContextValue] = useState({ container: undefined });
    useEffect(() => {
        const parent = parentContainer ?? params.parentContainer;
        if (!contextValue.container) {
            if (parent) {
                logger.log('create container');
                contextValue.container = new ReactContainer(parent);
            }
            else {
                logger.log('create root container');
                contextValue.container = new ReactContainer();
                contextValue.container.rerender = () => setContextValue({ ...contextValue });
            }
            setContextValue({ container: contextValue.container });
        }
        if (!parent || contextValue.container.sameParent(parent)) {
            return;
        }
        // exceptional case: if by some reasons parent container will be changed
        // possible, unreachable case
        logger.log('RECREATE container');
        setContextValue({ container: new ReactContainer(parent) });
    }, [parentContainer, params.parentContainer]);
    return contextValue;
}
//# sourceMappingURL=useDiContext.js.map